import unittest
from MoinMoin.util.header import Header


class LineIteratorTests(unittest.TestCase):

    def testEmptyText(self):
        """ util.header: lineIterator: empty text """
        header = Header(self.request, "")
        self.assertRaises(StopIteration, header.lineIterator().next)
        
    def testLines(self):
        """ util.header: lineIterator: iterate lines """
        header = Header(self.request, "##1\n##2\n")
        self.assertEqual([line for line in header.lineIterator()], 
                         ["##1\n", "##2\n"])

    def testMissingLineEnd(self):
        """ util.header: lineIterator: misssing trailing newline """
        header = Header(self.request, "##1")
        self.assertEqual([line for line in header.lineIterator()], 
                         ["##1"])
        

class SplitLineTests(unittest.TestCase):

    def setUp(self):
        self.header = Header(self.request, "")

    def testParseEmptyLine(self):
        """ util.header: splitLine: empty line """
        self.assertEqual(self.header.splitLine("\n"), (None, None))

    def testParseEmptyHeader(self):
        """ util.header: splitLine: empty header """
        header = Header(self.request, "")
        self.assertEqual(self.header.splitLine("#\n"), (None, None))

    def testParseComment(self):
        """ util.header: splitLine: comment """
        header = Header(self.request, "")
        self.assertEqual(self.header.splitLine("## Comment\n"), ('#', None))
        
    def testParseEmptyPI(self):
        """ util.header: splitLine: pi - empty """
        header = Header(self.request, "")
        self.assertEqual(self.header.splitLine("# pi value\n"), (None, None))

    def testValidPI(self):
        """ util.header: splitLine: pi - valid """
        header = Header(self.request, "")
        self.assertEqual(self.header.splitLine("#pi value\n"), ('pi', 'value'))

    def testPIWithoutValue(self):
        """ util.header: splitLine: pi - without value """
        header = Header(self.request, "")
        self.assertEqual(self.header.splitLine("#pi \n"), ('pi', ''))


class HeaderParsingTests(unittest.TestCase):

    def testEmptyHeader(self):
        """ util.header: empty headers """
        header = Header(self.request, "Empty header")
        self.assertEqual(header.length(), 0)

    def testEmptyHeaderWithPre(self):
        """ util.header: empty headers, comments in pre 
        
        Reported broken with Page.header_re by Thomas Werschlein
        """
        header = Header(self.request, "{{{\n# Comment\n}}}\n")
        self.assertEqual(header.length(), 0)

    def testIgnoreComments(self):
        """ util.header: ignore comments """
        header = Header(self.request, "##\nBody")
        expected = "##\n"
        self.assertEqual(header.length(), len(expected))

    def testNoLineEnding(self):
        """ util.header: missing line ending """
        header = Header(self.request, "#pi value")
        expected = "#pi value"
        self.assertEqual(header.length(), len(expected))

    def testBreakOnFirstNonHeader(self):
        """ util.header: break on first non-header """
        header = Header(self.request,
                        "#pi value\nbreak here!\n#pi value\n")
        expected = "#pi value\n"
        self.assertEqual(header.length(), len(expected))

    def testBreakOnInvalidPI(self):
        """ util.header: break on empty (invalid) pi """
        header = Header(self.request,
                        "#pi value\n# break here!\n#pi value\n")
        expected = "#pi value\n"
        self.assertEqual(header.length(), len(expected))

    def testProcessingInstruction(self):
        """ util.header: parse processing instruction """
        header = Header(self.request, "#pi value\nBody\n")
        expected = "#pi value\n"
        self.assertEqual(header.length(), len(expected))
        self.assertEqual(header.get('pi'), 'value')

    def testEmptyProcessingInstruction(self):
        """ util.header: empty processing instruction saved """
        header = Header(self.request, "#pi   \nBody\n")
        expected = "#pi   \n"
        self.assertEqual(header.length(), len(expected))
        self.assertEqual(header.get('pi'), '')
   
    def testPragma(self):
        """ util.header: parse processing instruction """
        header = Header(self.request, "#pragma key value\nBody\n")
        expected = "#pragma key value\n"
        self.assertEqual(header.length(), len(expected))
        self.assertEqual(header.get('pragma').get('key'), 'value')

    def testPragmaWithoutValue(self):
        """ util.header: ignore pragma with no value """
        header = Header(self.request, "#pragma key    \nBody\n")
        expected = "#pragma key    \n"
        self.assertEqual(header.length(), len(expected))
        self.assertEqual(header.get('pragma'), None)

    def testEmptyPragma(self):
        """ util.header: ignore empty pragma """
        header = Header(self.request, "#pragma  \nBody\n")
        expected = "#pragma  \n"
        self.assertEqual(header.length(), len(expected))
        self.assertEqual(header.get('pragma'), None)
        
    def testStripWhitespace(self):
        """ util.header: strip white space around tokens """
        header = Header(self.request,
                        "#pi  value  \n#pragma  key  value  \nBody\n")
        self.assertEqual(header.get('pi'), 'value')
        self.assertEqual(header.get('pragma').get('key'), 'value')

    def testACL(self):
        """ util.header: collect acl lines """
        header = Header(self.request, "#acl line1\n#acl line2\nBody\n")
        self.assertEqual(header.get('acl'), ["line1", "line2"])

    def testIgnoreEmptyACL(self):
        """ util.header: ignore empty acl lines """
        header = Header(self.request, "#acl line1\n#acl \n#acl line2\nBody\n")
        self.assertEqual(header.get('acl'), ["line1", "line2"])

    def testKnownLanguage(self):
        """ util.header: save known language """
        header = Header(self.request, "#language en\nBody\n")
        self.assertEqual(header.get('language'), 'en')
        
    def testUnknownLanguage(self):
        """ util.header: ignore unknown language """
        header = Header(self.request, "#language he-unkown\nBody\n")
        self.assertEqual(header.get('language'), None)


class UnkownHeaderTests(unittest.TestCase):
    
    class PageHeader(Header):        
        def isAllowed(self, key):
            return False
            
    def testUnknownHeader(self):
        """ util.header: break on unknown header (1.3 behavior) """
        header = self.PageHeader(self.request, "#unknown value\nBody\n")
        self.assertEqual(header.length(), 0)
        self.assertEqual(header.get('unknown'), None)
