"""
MoinMoin header
===============

Header represent wiki page header, which include comments, processing
instructions and pragmas. The header parse text and save all valid
header in the text. Headers are accessed like a dict.

If you want to add custom handling of certain headers, subclass and
add set_xxx methods. The method will be called by the parser for each
found header with that name. To change the default handling of headers,
override setHeader.

Usage
-----
::
    header = Header(request, text)
    # This will triger parsing of text
    language = header['language']

@copyright: 2005 Nir Soffer <nirs@freeshell.org>
@license: GNU GPL, see COPYING for details.
"""

from __future__ import generators
import re
from MoinMoin import i18n


class Header(object):
    """ Immutable wiki page header

    Header represent a wiki page header, containing comments,
    processing instructinos and pragmas.
    
    Header parsing is done lazily, on the first accessor call.
    """
    # -----------------------------------------------------------------
    # Creation 

    def __init__(self, request, text):
        self.request = request
        self.text = text
        self._headers = None
        self._length = None
    
    # -----------------------------------------------------------------
    # Accessing

    # Header is accessed like a dict.
    
    def __getitem__(self, key):
        """ dict style obj[key] handler """
        return self.headers()[key]

    def get(self, key, default=None):
        """ dict style obj.get(key) handler """
        return self.headers().get(key, default)

    def __contains__(self, key):
        """ in obj handler """
        return key in self.headers()
    
    def length(self):
        """ Return length of page header characters
        
        Trigger parsing on first call.
        """
        if self._length is None:
            self.parse()
        return self._length

    # -----------------------------------------------------------------
    # Private

    # Methods here are private, but does not use the _convetion becasue
    # it is less readable.
    
    def headers(self):
        """ Return headers dict, trigger parsing on first call """
        if self._headers is None:
            self.parse()
        return self._headers

    def parse(self):
        """ Start parsing, triggered automatically 
        
        Duplicate moin 1.3 parsing behavior: break on first unknown
        header.
        """
        self._headers = {}
        self._length = 0
        for line in self.lineIterator():
            key, value = self.splitLine(line)
            if not key:
                break
            if value is not None:
                setKey = getattr(self, 'set_' + key, None)
                if setKey:
                    setKey(value)
                elif self.isAllowed(key):
                    self.setHeader(key, value)
                else:
                    break            
            self._length += len(line)

    def lineIterator(self):
        """ Return an efficient iterator over lines
        
        Require using '\n' line endings (enforced by Page). Lines
        include the line ending.
        """
        start = 0
        last = len(self.text)
        while start < last:
            nextLine = self.text.find('\n', start) + 1
            if nextLine == 0:
                nextLine = last
            yield self.text[start:nextLine] 
            start = nextLine

    def splitLine(self, line):
        """ Parse line and return key and value
        
        Implements header syntax::
        
            #key value
            ## Comment (ignored)
        
        Duplicate moin 1.3 parsing: break on first non header line or
        empty (invalid) header. 
        """
        if not line.startswith('#') or line == '#' or line[1].isspace():
            return None, None
        if line.startswith('##'):
            return '#', None
        return self.splitTokens(line[1:])

    def splitTokens(self, text):
        """ Split first token and rest of text by whitespace """
        try:
            key, value = text.split(None, 1)
            return key, value.rstrip()
        except ValueError:
            return text.strip(), ''
            
    def isAllowed(self, key):
        """ Override to allow only certain headers. Default allow any """
        return True

    def set_acl(self, value):
        """ Save all non-empty acl lines keeping the order """
        if value:
            self.appendHeader('acl', value)
    
    def set_language(self, value):
        """ Save known languages, ignore other """
        if value in i18n.wikiLanguages():
            self.setHeader('language', value)
            
    def set_pragma(self, text):
        """ Save pragmas, ignoring empty pragmas """
        key, value = self.splitTokens(text)
        if value:
            try:
                self._headers['pragma'][key] = value
            except KeyError:
                self._headers['pragma'] = {key: value}
                                                                
    def setHeader(self, key, value):
        """ Set header value, last value override previous values """
        self._headers[key] = value    
    
    def appendHeader(self, key, value):
        """ Append header value, keep all values found """
        try:
            self._headers[key].append(value)
        except KeyError:
            self._headers[key] = [value]
            